package com.hexin.android.component.listview

import android.util.SparseArray
import androidx.recyclerview.widget.RecyclerView
import android.view.View
import android.view.ViewGroup

abstract class ListAdapter<T : ListItemViewHolder> : RecyclerView.Adapter<T>() {

    private val mOnClickListener = View.OnClickListener {
        onItemClickListener?.onItemClick(it.tag as Int)
    }

    var onItemClickListener: OnItemClickListener? = null

    abstract fun onCreateView(parent: ViewGroup?, viewType: Int): View

    abstract fun onCreateViewHolder(convertView: View, viewType: Int): T

    abstract fun onBindViewHolder(parent: ViewGroup?, holder: T?, position: Int)

    open fun onCreateHeaderOrFooterViewHolder(convertView: View, viewType: Int): T {
        return onCreateViewHolder(convertView, viewType)
    }

    open fun onBindHeaderOrFooterViewHolder(parent: ViewGroup?, holder: T?, type: Int) {
    }

    open fun getDataViewType(position: Int): Int {
        return super.getItemViewType(position)
    }

    abstract fun getDataCount(): Int

    final override fun getItemCount(): Int {
        return getDataCount() + mFooterViewCount + mHeaderViewCount
    }

    final override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): T {
        val convertView = when {
            (mHeaderViews[viewType] != null) -> {
                mHeaderViews[viewType]
            }
            (mFooterViews[viewType] != null) -> {
                mFooterViews[viewType]
            }
            else -> onCreateView(parent, viewType)
        }

        val viewHolder = when {
            (mHeaderViews[viewType] != null || mFooterViews[viewType] != null) -> {
                onCreateHeaderOrFooterViewHolder(convertView, viewType)
            }
            else -> {
                convertView.setOnClickListener(mOnClickListener)
                onCreateViewHolder(convertView, viewType)
            }
        }

        viewHolder.parent = parent

        return viewHolder
    }

    final override fun onBindViewHolder(holder: T, position: Int) {
        val headerFooterViewType = when {
            isHeaderViewPos(position) -> {
                mHeaderViews.keyAt(position)
            }
            isFooterViewPos(position) -> {
                mFooterViews.keyAt(position - mHeaderViewCount - getDataCount())
            }
            else -> {
                holder.itemView.tag = position - mHeaderViewCount
                null
            }
        }
        updateSlidingPosition(holder)
        when {
            (headerFooterViewType != null) -> {
                onBindHeaderOrFooterViewHolder(holder.parent, holder, headerFooterViewType)
            }
            else -> onBindViewHolder(holder.parent, holder, position - mHeaderViewCount)
        }
    }

    private fun updateSlidingPosition(holder: T?) {
        val slidingItem = holder?.itemView as? SlidingItem ?: return
        val slidingRecyclerView = holder.parent as? SlidingInfoPicker ?: return
        slidingRecyclerView.getScrolledX()?.let { slidingItem.getSlidingView()?.scrollTo(it, 0) }
    }

    companion object {
        private const val BASE_ITEM_TYPE_HEADER = 100000
        private const val BASE_ITEM_TYPE_FOOTER = 200000
    }

    private val mHeaderViews = SparseArray<View>()
    private val mFooterViews = SparseArray<View>()

    private val mHeaderViewCount: Int
        get() = mHeaderViews.size()
    private val mFooterViewCount: Int
        get() = mFooterViews.size()

    final override fun getItemViewType(position: Int): Int {
        return when {
            (isHeaderViewPos(position)) -> {
                mHeaderViews.keyAt(position)
            }
            (isFooterViewPos(position)) -> {
                mFooterViews.keyAt(position - mHeaderViewCount - getDataCount())
            }
            else -> getDataViewType(position - mHeaderViewCount)
        }
    }

    fun addHeaderView(headerView: View?) {
        addHeaderView(headerView, mHeaderViewCount)
    }

    fun addHeaderView(headerView: View?, type: Int) {
        headerView ?: return
        mHeaderViews.put(type + BASE_ITEM_TYPE_HEADER, headerView)
    }

    fun addFooterView(footerView: View?) {
        addFooterView(footerView, mFooterViewCount)
    }

    fun addFooterView(footerView: View?, type: Int) {
        footerView ?: return
        mFooterViews.put(type + BASE_ITEM_TYPE_FOOTER, footerView)
    }

    fun removeFooterView(key: Int) {
        mFooterViews.remove(key)
    }

    fun removeAllFooterView() {
        mFooterViews.clear()
    }

    private fun isHeaderViewPos(position: Int): Boolean {
        return mHeaderViewCount > position
    }

    private fun isFooterViewPos(position: Int): Boolean {
        return mHeaderViewCount + getDataCount() <= position
    }

}
